@@ -44,12 +44,6 @@
 #undef  TIMES_2
 #undef  GETRUSAGE_2
 #define HAVE_ALLOCA
-/* vc 2005 */
-#if defined(_MSC_VER) && _MSC_VER >= 1400
-#	define HAVE_VSNPRINTF
-#	define HAVE_SNPRINTF
-#	define snprintf _snprintf
-#endif
 #define SPATH "."
 #define DPATH "."
 #define NO_U_TYPES
@@ -73,4 +67,17 @@
 #define intptr_t long
 #endif
 
+/* Generation version number (e.g.: 3 for Unreal3*) */
+#define UNREAL_VERSION_GENERATION 3
+
+/* Major version number (e.g.: 2 for Unreal3.2*) */
+#define UNREAL_VERSION_MAJOR 2
+
+/* Minor version number (e.g.: 1 for Unreal3.2.1) */
+#define UNREAL_VERSION_MINOR 10
+
+/* Version suffix such as a beta marker or release candidate marker. (e.g.:
+   -rc2 for unrealircd-3.2.9-rc2) */
+#define UNREAL_VERSION_SUFFIX ""
+
 #endif
