@@ -61,7 +61,7 @@ ModuleHeader l_commands_Header
 #endif
   = {
 	"commands",	/* Name of module */
-	"3.2.9", /* Version */
+	"3.2.10", /* Version */
 	"Wrapper library for m_ commands", /* Short description of module */
 	"3.2-b8-1",
 	NULL 
@@ -126,6 +126,8 @@ extern int m_svsnolag_Init(ModuleInfo *modinfo);
 extern int m_starttls_Init(ModuleInfo *modinfo);
 extern int m_nopost_Init(ModuleInfo *modinfo);
 extern int m_issecure_Init(ModuleInfo *modinfo);
+extern int m_cap_Init(ModuleInfo *modinfo);
+extern int m_sasl_Init(ModuleInfo *modinfo);
 #ifdef GUEST
 extern int m_guest_Init(ModuleInfo *modinfo);
 #endif
@@ -178,6 +180,8 @@ extern int m_svsnolag_Load(int module_load);
 extern int m_starttls_Load(int module_load);
 extern int m_nopost_Load(int module_load);
 extern int m_issecure_Load(int module_load);
+extern int m_cap_Load(int module_load);
+extern int m_sasl_Load(int module_load);
 #ifdef GUEST
 extern int m_guest_Load(int module_load);
 #endif
@@ -217,6 +221,8 @@ extern int m_lusers_Unload(), m_names_Unload(), m_svsnolag_Unload();
 extern int m_starttls_Unload();
 extern int m_nopost_Unload();
 extern int m_issecure_Unload();
+extern int m_cap_Unload();
+extern int m_sasl_Unload();
 #ifdef GUEST
 extern int m_guest_Unload();
 #endif
@@ -362,6 +368,8 @@ int    l_commands_Init(ModuleInfo *modinfo)
 	m_starttls_Init(ModCmdsInfo);
 	m_nopost_Init(ModCmdsInfo);
 	m_issecure_Init(ModCmdsInfo);
+	m_cap_Init(ModCmdsInfo);
+	m_sasl_Init(ModCmdsInfo);
 #ifdef GUEST
 	m_guest_Init(ModCmdsInfo);
 #endif
@@ -477,6 +485,8 @@ int    l_commands_Load(int module_load)
 	m_starttls_Load(module_load);
 	m_nopost_Load(module_load);
 	m_issecure_Load(module_load);
+	m_cap_Load(module_load);
+	m_sasl_Load(module_load);
 #ifdef GUEST
 	m_guest_Load(module_load);
 #endif
@@ -592,6 +602,8 @@ int	l_commands_Unload(int module_unload)
 	m_starttls_Unload();
 	m_nopost_Unload();
 	m_issecure_Unload();
+	m_cap_Unload();
+	m_sasl_Unload();
 #ifdef GUEST
 	m_guest_Unload();
 #endif
