@@ -692,22 +692,24 @@ DLLFUNC CMD_FUNC(m_nick)
 	}
 	else if (!sptr->name[0])
 	{
-#ifdef NOSPOOF
-		/*
-		 * Client setting NICK the first time.
-		 *
-		 * Generate a random string for them to pong with.
-		 */
-		sptr->nospoof = getrandom32();
+		if (iConf.ping_cookie)
+		{
+			/*
+			 * Client setting NICK the first time.
+			 *
+			 * Generate a random string for them to pong with.
+			 */
+			sptr->nospoof = getrandom32();
 
-		if (PINGPONG_WARNING)
-			sendto_one(sptr, ":%s NOTICE %s :*** If you are having problems"
-			    " connecting due to ping timeouts, please"
-			    " type /quote pong %X or /raw pong %X now.",
-			    me.name, nick, sptr->nospoof, sptr->nospoof);
+			if (PINGPONG_WARNING)
+				sendto_one(sptr, ":%s NOTICE %s :*** If you are having problems"
+				    " connecting due to ping timeouts, please"
+				    " type /quote pong %X or /raw pong %X now.",
+				    me.name, nick, sptr->nospoof, sptr->nospoof);
+
+			sendto_one(sptr, "PING :%X", sptr->nospoof);
+		}
 
-		sendto_one(sptr, "PING :%X", sptr->nospoof);
-#endif /* NOSPOOF */
 #ifdef CONTACT_EMAIL
 		sendto_one(sptr,
 		    ":%s NOTICE %s :*** If you need assistance with a"
@@ -736,7 +738,7 @@ DLLFUNC CMD_FUNC(m_nick)
 		}
 		/* This had to be copied here to avoid problems.. */
 		(void)strcpy(sptr->name, nick);
-		if (sptr->user && IsNotSpoof(sptr))
+		if (sptr->user && IsNotSpoof(sptr) && !CHECKPROTO(sptr, PROTO_CLICAP))
 		{
 			/*
 			   ** USER already received, now we have NICK.
@@ -746,11 +748,10 @@ DLLFUNC CMD_FUNC(m_nick)
 			   ** may reject the client and call exit_client for it
 			   ** --must test this and exit m_nick too!!!
 			 */
-#ifndef NOSPOOF
-			if (USE_BAN_VERSION && MyConnect(sptr))
+			if (!iConf.ping_cookie && USE_BAN_VERSION && MyConnect(sptr))
 				sendto_one(sptr, ":IRC!IRC@%s PRIVMSG %s :\1VERSION\1",
 					me.name, nick);
-#endif
+
 			sptr->lastnick = TStime();	/* Always local client */
 			if (register_user(cptr, sptr, nick,
 			    sptr->user->username, NULL, NULL, NULL) == FLUSH_BUFFER)
@@ -1100,7 +1101,6 @@ int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, cha
 			    sptr->name, olduser, userbad, stripuser);
 #endif
 		nextping = TStime();
-		sendto_connectnotice(nick, user, sptr, 0, NULL);
 		if (IsSecure(sptr))
 			sptr->umodes |= UMODE_SECURE;
 	}
@@ -1170,17 +1170,20 @@ int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, cha
 	/* NICKv2 Servers ! */
 	sendto_serv_butone_nickcmd(cptr, sptr, nick,
 	    sptr->hopcount + 1, sptr->lastnick, user->username, user->realhost,
-	    user->server, user->servicestamp, sptr->info,
+	    user->server, user->svid, sptr->info,
 	    (!buf || *buf == '\0' ? "+" : buf),
 	    sptr->umodes & UMODE_SETHOST ? sptr->user->virthost : NULL);
 
-	/* Send password from sptr->passwd to NickServ for identification,
-	 * if passwd given and if NickServ is online.
-	 * - by taz, modified by Wizzu
-	 */
 	if (MyConnect(sptr))
 	{
 		char userhost[USERLEN + HOSTLEN + 6];
+
+		sendto_connectnotice(nick, user, sptr, 0, NULL); /* moved down, for modules. */
+
+		/* Send password from sptr->passwd to NickServ for identification,
+		 * if passwd given and if NickServ is online.
+		 * - by taz, modified by Wizzu
+		 */
 		if (sptr->passwd && (nsptr = find_person(NickServ, NULL)))
 		{
 			int do_identify = 1;
