@@ -39,6 +39,10 @@ static char sccsid[] =
 Computing Center and Jarkko Oikarinen";
 #endif
 
+#ifdef _WIN32
+#include <WinSock2.h>
+#endif
+
 #include "struct.h"
 #include "common.h"
 #include "sys.h"
@@ -69,6 +73,23 @@ Computing Center and Jarkko Oikarinen";
 #include  "fdlist.h"
 #endif
 
+#ifdef USE_POLL
+# ifndef _WIN32
+#  include <sys/poll.h>
+#  include <poll.h>
+#  ifndef POLLRDHUP
+#   define POLLRDHUP 0
+#  endif
+# else
+#  define poll WSAPoll
+#  define POLLRDHUP POLLHUP
+# endif
+
+static struct pollfd pollfds[MAXCONNECTIONS], dummy_pollfd;
+static int pollfd_count = 0;
+static int pollfd_to_client[MAXCONNECTIONS]; /* use get_client_by_pollfd() for grabbing, don't grab from array directly ! */
+#endif
+
 #ifdef INET6
 static unsigned char minus_one[] =
     { 255, 255, 255, 255, 255, 255, 255, 255, 255,
@@ -138,6 +159,60 @@ extern void url_do_transfers_async(void);
 #  endif
 # endif
 #endif
+
+#ifdef USE_POLL
+static void reset_pollfd()
+{
+	int i;
+	
+	pollfd_count = 0;
+	for (i = 0; i < MAXCONNECTIONS; ++i)
+		pollfd_to_client[i] = -1;
+}
+#define POLL_RESOLVER -2
+/** Return a pollfd structure and map file descriptor (fd) to client slot (slot).
+ * Although we do some minimal MAXCONNECTIONS checking here to ensure we don't crash,
+ * the caller must still check if fd >= MAXCONNECTIONS as this is a serious issue.
+ */
+static struct pollfd *get_pollfd(int slot, int fd)
+{
+	if (pollfd_count >= MAXCONNECTIONS)
+	{
+		ircd_log(LOG_ERROR, "TROUBLE: get_pollfd() called with slot %d fd %d, which is >=%d. This is bad!",
+			slot, fd, MAXCONNECTIONS);
+#ifdef DEBUGMODE
+		abort(); /* I think this warrants a core dump in debug mode. -- Syzop */
+#endif
+		memset(&dummy_pollfd, 0, sizeof(dummy_pollfd));
+		return &dummy_pollfd;
+	}
+	else
+	{
+		struct pollfd *p = &pollfds[pollfd_count++];
+		memset(p, 0, sizeof(*p));
+		p->fd = fd;
+		pollfd_to_client[fd] = slot;
+		return p;
+	}
+}
+
+#if 1
+/* For now, always use this function which does extra checking.. paranoid.. -- Syzop */
+static int get_client_by_pollfd(int fd)
+{
+int v;
+	if ((fd < 0) || (fd >= MAXCONNECTIONS))
+		abort();
+	v = pollfd_to_client[fd];
+	/* v may be negative, both when it's POLL_RESOLVER and during a race condition */
+	return v;
+}
+#else
+#define get_client_by_pollfd(x) pollfd_to_client[x]
+#endif /* DEBUGMODE */
+
+#endif /* USE_POLL */
+
 void start_of_normal_client_handshake(aClient *acptr);
 void proceed_normal_client_handshake(aClient *acptr, struct hostent *he);
 
@@ -170,6 +245,14 @@ void remove_local_client(aClient* cptr)
 		return;
 	}
 
+#ifdef USE_POLL
+	/* Using the pollfd_to_client array directly here, as we do proper bounds checks ! */
+	if ((cptr->fd >= 0) && (cptr->fd < MAXCLIENTS) && (pollfd_to_client[cptr->fd] == cptr->slot))
+		pollfd_to_client[cptr->fd] = -1;
+	if ((cptr->authfd >= 0) && (cptr->authfd < MAXCLIENTS) && (pollfd_to_client[cptr->authfd] == cptr->slot))
+		pollfd_to_client[cptr->authfd] = -1;
+#endif
+
 	/* Keep LastSlot as the last one
 	 */
 	local[cptr->slot] = NULL;
@@ -528,7 +611,7 @@ void close_listeners(void)
  */
 void init_sys(void)
 {
-	int  fd;
+	int  fd, i;
 #ifdef RLIMIT_FD_MAX
 	struct rlimit limit;
 
@@ -563,12 +646,14 @@ void init_sys(void)
 }
 #endif
 #ifndef _WIN32
+#ifndef USE_POLL
 	if (MAXCONNECTIONS > FD_SETSIZE)
 	{
 		fprintf(stderr, "MAXCONNECTIONS (%d) is higher than FD_SETSIZE (%d)\n", MAXCONNECTIONS, FD_SETSIZE);
 		fprintf(stderr, "You might need to recompile the IRCd, or if you're running Linux, read the release notes\n");
 		exit(-1);
 	}
+#endif
 #endif
 	/* Startup message
 	   pid = getpid();
@@ -652,6 +737,9 @@ init_dgram:
 openlog("ircd", LOG_PID | LOG_NDELAY, LOG_DAEMON); /* reopened now */
 #endif
 	memset(local, 0, sizeof(aClient*) * MAXCONNECTIONS);
+#ifdef USE_POLL
+	reset_pollfd();
+#endif
 	LastSlot = -1;
 
 #endif /*_WIN32*/
@@ -1421,14 +1509,18 @@ void proceed_normal_client_handshake(aClient *acptr, struct hostent *he)
 ** chunks to give a better performance rating (for server connections).
 ** Do some tricky stuff for client connections to make sure they don't do
 ** any flooding >:-) -avalon
+** If 'doread' is set to 0 then we don't actually read (no recv()),
+** however we still check if we need to dequeue anything from the recvQ.
+** This is necessary, since we may have put something on the recvQ due
+** to fake lag. -- Syzop
 */
 
-static int read_packet(aClient *cptr, fd_set *rfd)
+static int read_packet(aClient *cptr, int doread)
 {
 	int  dolen = 0, length = 0, done;
 	time_t now = TStime();
-	if (FD_ISSET(cptr->fd, rfd) &&
-	    !(IsPerson(cptr) && DBufLength(&cptr->recvQ) > 6090))
+	
+	if (doread && !(IsPerson(cptr) && DBufLength(&cptr->recvQ) > 6090))
 	{
 		Hook *h;
 		SET_ERRNO(0);
@@ -1473,7 +1565,7 @@ static int read_packet(aClient *cptr, fd_set *rfd)
 		   ** it on the end of the receive queue and do it when its
 		   ** turn comes around.
 		 */
-		if (!dbuf_put(&cptr->recvQ, readbuf, length))
+		if (length && !dbuf_put(&cptr->recvQ, readbuf, length))
 			return exit_client(cptr, cptr, cptr, "dbuf_put fail");
 
 		if (IsPerson(cptr) && DBufLength(&cptr->recvQ) > get_recvq(cptr))
@@ -1553,13 +1645,20 @@ int  read_message(time_t delay, fdlist *listp)
    #undef FD_SET(x,y) do { if (fcntl(x, F_GETFD, &sockerr) == -1) abort(); FD_SET(x,y); } while(0)
 */	aClient *cptr;
 	int  nfds;
+#ifdef USE_POLL
+	struct pollfd *pfd;
+	int poll_fd_count = 0;
+	ares_socket_t aresfds[ARES_GETSOCK_MAXNUM];
+#else
 	struct timeval wait;
+#endif
+
 #ifndef _WIN32
 	fd_set read_set, write_set;
 #else
 	fd_set read_set, write_set, excpt_set;
 #endif
-	int  j,k;
+	int  j,k, v;
 	time_t delay2 = delay, now;
 	int  res, length, fd, i;
 	int  auth = 0;
@@ -1583,9 +1682,14 @@ int  read_message(time_t delay, fdlist *listp)
 #ifdef _WIN32
 		FD_ZERO(&excpt_set);
 #endif
+#ifdef USE_POLL
+		reset_pollfd();
+#endif /* USE_POLL */
+
 #ifdef USE_LIBCURL
 		url_do_transfers_async();
 #endif
+
 #ifdef NO_FDLIST
 		for (i = LastSlot; i >= 0; i--)
 #else
@@ -1613,12 +1717,24 @@ int  read_message(time_t delay, fdlist *listp)
 					Debug((DEBUG_NOTICE, "auth on %x %d %d", cptr, i, s));
 					if (cptr->authfd >= 0)
 					{
+#ifdef USE_POLL
+						pfd = get_pollfd(cptr->slot, cptr->authfd);
+						pfd->events = POLLIN;
+#else
 						FD_SET(cptr->authfd, &read_set);
 #ifdef _WIN32	
 						FD_SET(cptr->authfd, &excpt_set);
 #endif	
+#endif
+
 						if (cptr->flags & FLAGS_WRAUTH)
+						{
+#ifdef USE_POLL
+							pfd->events |= POLLOUT;
+#else
 							FD_SET(cptr->authfd, &write_set);
+#endif
+						}
 					}
 				}
 			}
@@ -1627,17 +1743,36 @@ int  read_message(time_t delay, fdlist *listp)
 			 */
 			if (DoingDNS(cptr) || DoingAuth(cptr))
 				continue;
+#ifdef USE_POLL
+			pfd = NULL;
+#endif
 			if (IsMe(cptr) && IsListening(cptr))
 			{
 				if (cptr->fd >= 0)
+				{
+#ifdef USE_POLL
+					if (pfd == NULL)
+						pfd = get_pollfd(cptr->slot, cptr->fd);
+					pfd->events |= POLLIN;
+#else
 					FD_SET(cptr->fd, &read_set);
+#endif
+				}
 			}
 			else if (!IsMe(cptr))
 			{
 				if (DBufLength(&cptr->recvQ) && delay2 > 2)
 					delay2 = 1;
 				if ((cptr->fd >= 0) && (DBufLength(&cptr->recvQ) < 4088))
+				{
+#ifdef USE_POLL
+					if (pfd == NULL)
+						pfd = get_pollfd(cptr->slot, cptr->fd);
+					pfd->events |= POLLIN;
+#else
 					FD_SET(cptr->fd, &read_set);
+#endif
+				}
 			}
 			if ((cptr->fd >= 0) && (DBufLength(&cptr->sendQ) || IsConnecting(cptr) ||
 			    (DoList(cptr) && IsSendable(cptr))
@@ -1646,15 +1781,51 @@ int  read_message(time_t delay, fdlist *listp)
 #endif
 			    ))
 			{
+#ifdef USE_POLL
+				if (pfd == NULL)
+					pfd = get_pollfd(cptr->slot, cptr->fd);
+				pfd->events |= POLLOUT;
+#else
 				FD_SET(cptr->fd, &write_set);
+#endif
 			}
 		}
 
-		ares_fds(resolver_channel, &read_set, &write_set);
-		
-		if (me.fd >= 0)
-			FD_SET(me.fd, &read_set);
+#ifdef USE_POLL
+		memset(&aresfds, 0, sizeof(aresfds));
+		v = ares_getsock(resolver_channel, aresfds, ARES_GETSOCK_MAXNUM);
+		for (k = 0; k < ARES_GETSOCK_MAXNUM; k++)
+		{
+			pfd = NULL;
+			if (ARES_GETSOCK_READABLE(v, k))
+			{
+				pfd = get_pollfd(POLL_RESOLVER, aresfds[k]);
+				pfd->events |= POLLIN;
+			}
+			if (ARES_GETSOCK_WRITABLE(v, k))
+			{
+				if (!pfd)
+					pfd = get_pollfd(POLL_RESOLVER, aresfds[k]);
+				pfd->events |= POLLOUT;
+			}
+		}
+#else
+		nfds = ares_fds(resolver_channel, &read_set, &write_set);
+#endif
 
+		if (me.fd >= 0)
+		{
+#ifdef USE_POLL
+			pfd = get_pollfd(me.slot, me.fd);
+			pfd->events |= POLLIN;
+#else
+			FD_SET(me.fd, &read_set);
+#endif
+		}
+
+#ifdef USE_POLL
+		nfds = poll(pollfds, pollfd_count, MIN(delay, delay2) * 1000);
+#else /* USE_POLL */
 		wait.tv_sec = MIN(delay, delay2);
 		wait.tv_usec = 0;
 #ifdef	HPUX
@@ -1667,6 +1838,7 @@ int  read_message(time_t delay, fdlist *listp)
 		nfds = select(MAXCONNECTIONS, &read_set, &write_set, &excpt_set, &wait);
 # endif
 #endif
+#endif /* USE_POLL */
 	    if (nfds == -1 && ((ERRNO == P_EINTR) || (ERRNO == P_ENOTSOCK)))
 			return -1;
 		else if (nfds >= 0)
@@ -1683,22 +1855,56 @@ int  read_message(time_t delay, fdlist *listp)
 	}
 
 	Debug((DEBUG_DNS, "Doing DNS async.."));
+#ifndef USE_POLL
 	ares_process(resolver_channel, &read_set, &write_set);
+#else
+	/* Ensure that ares processing gets called at least once every loop,
+	 * so it can handle timeouts. -- Syzop
+	 */
+	ares_process_fd(resolver_channel, ARES_SOCKET_BAD, ARES_SOCKET_BAD);
+#endif
 
 	/*
 	 * Check fd sets for the auth fd's (if set and valid!) first
 	 * because these can not be processed using the normal loops below.
 	 * -avalon
+	 * UPDATE: if using poll, we now also use this one for our resolver
+	 *         (c-ares). -- Syzop
 	 */
+#ifdef USE_POLL
+	for (i = 0; i < pollfd_count; ++i)
+#else
 #ifdef NO_FDLIST
 	for (i = LastSlot; (auth > 0) && (i >= 0); i--)
 #else
 	for (i = listp->entry[j = 1]; j <= listp->last_entry; i = listp->entry[++j])
 #endif
+#endif /* USE_POLL */
 	{
-		if (!(cptr = local[i]))
+#ifdef USE_POLL
+		pfd = &pollfds[i];
+		v = get_client_by_pollfd(pfd->fd);
+		if (v == POLL_RESOLVER)
+		{
+			/* Handle resolver stuff */
+			int rdfd = ARES_SOCKET_BAD;
+			int wrfd = ARES_SOCKET_BAD;
+			if (pfd->revents & POLLIN)
+				rdfd = pfd->fd;
+			if (pfd->revents & POLLOUT)
+				wrfd = pfd->fd;
+			ares_process_fd(resolver_channel, rdfd, wrfd);
+			/* handled, continue. */
+			nfds--;
 			continue;
-		if (cptr->authfd < 0)
+		}
+		if (v < 0)
+			continue; /* Hm, shouldn't happen, but still.. */
+		cptr = local[v];
+#else
+		cptr = local[i];
+#endif
+		if (!cptr || cptr->authfd < 0)
 			continue;
 		auth--;
 #ifdef _WIN32
@@ -1707,7 +1913,11 @@ int  read_message(time_t delay, fdlist *listp)
 		 * the exception FD set to find out when a connection is
 		 * refused.  ie Auth ports and /connect's.  -Cabal95
 		 */
+#ifdef USE_POLL
+		if (pfd->revents & (POLLERR | POLLRDHUP))
+#else
 		if (FD_ISSET(cptr->authfd, &excpt_set))
+#endif
 		{
 			int  err, len = sizeof(err);
 
@@ -1729,29 +1939,53 @@ int  read_message(time_t delay, fdlist *listp)
 #endif
 		if (nfds > 0)
 		{
+#ifdef USE_POLL
+			if (pfd->revents & (POLLIN | POLLOUT))
+#else
 			if (FD_ISSET(cptr->authfd, &read_set) ||
 				FD_ISSET(cptr->authfd, &write_set))
+#endif
 				nfds--;
-			if ((cptr->authfd > 0) && FD_ISSET(cptr->authfd, &write_set))
-			{
+#ifdef USE_POLL
+			if ((cptr->authfd > 0) && (pfd->revents & POLLOUT))
 				send_authports(cptr);
-			}
-			if ((cptr->authfd > 0) && FD_ISSET(cptr->authfd, &read_set))
-			{
+			if ((cptr->authfd > 0) && (pfd->revents & POLLIN))
 				read_authports(cptr);
-			}
+#else
+			if ((cptr->authfd > 0) && (FD_ISSET(cptr->authfd, &write_set)))
+				send_authports(cptr);
+			if ((cptr->authfd > 0) && (FD_ISSET(cptr->authfd, &read_set)))
+				read_authports(cptr);
+#endif
 		}
 	}
 
+#ifdef USE_POLL
+	for (i = 0; i < pollfd_count; ++i)
+#else
 #ifdef NO_FDLIST
 	for (i = LastSlot; i >= 0; i--)
 #else
 	for (i = listp->entry[j = 1];  (j <= listp->last_entry); i = listp->entry[++j])
 #endif
-		if ((cptr = local[i]) && FD_ISSET(cptr->fd, &read_set) &&
+#endif /* USE_POLL */
+	{
+#ifdef USE_POLL
+		pfd = &pollfds[i];
+		v = get_client_by_pollfd(pfd->fd);
+		if (v < 0)
+			continue; /* possible with resolver */
+		cptr = local[v];
+		if (cptr && pfd->revents & POLLIN &&
+#else
+		cptr = local[i];
+		if (cptr && FD_ISSET(cptr->fd, &read_set) &&
+#endif
 		    IsListening(cptr))
 		{
+#ifndef USE_POLL
 			FD_CLR(cptr->fd, &read_set);
+#endif
 			nfds--;
 			cptr->lasttime = TStime();
 			/*
@@ -1775,7 +2009,26 @@ int  read_message(time_t delay, fdlist *listp)
 				break;
 			}
 			ircstp->is_ac++;
-			if (++OpenFiles >= MAXCLIENTS)
+			/* We now check:
+			 *  1.  The number of open files, which is the limit imposed by the
+			 *      user during ./Config (MAXCONNECTIONS minus a few).
+			 *  2a) When using select(): 
+			 *      If the fd number exceeds FD_SETSIZE, which is not
+			 *      permitted as otherwise FD_SET() and FD_CLR() will fail
+			 *      in read_message()
+			 *      Note that the value of FD_SETSIZE may be (much) higher than
+			 *      MAXCLIENTS/MAXCONNECTIONS. They are not necessarily the same!
+			 *      FIXME: Figure out why I need a FD_SETSIZE-4 here? still have crashes with -1.
+			 *  2b) When using poll():
+			 *      Similar to 2a, check if fd is within 0..MAXCLIENTS bounds, because we
+			 *      use pollfd_to_client[] to map fd's to client slots.
+ 			 */
+			if ((++OpenFiles >= MAXCLIENTS) ||
+#ifdef USE_POLL
+			    (fd >= MAXCLIENTS))
+#else
+ 			    (fd > FD_SETSIZE-4))
+#endif
 			{
 				ircstp->is_ref++;
 				if (last_allinuse < TStime() - 15)
@@ -1807,16 +2060,35 @@ int  read_message(time_t delay, fdlist *listp)
 				cptr->listener = &me;
 		        
                       }
+	}
+
+#ifdef USE_POLL
+	for (i = 0; i < pollfd_count; ++i)
+#else
 #ifndef NO_FDLIST
 	for (i = listp->entry[j = 1];  (j <= listp->last_entry); i = listp->entry[++j])
 #else
 	for (i = LastSlot; i >= 0; i--)
+#endif
 #endif
 	{
-		if (!(cptr = local[i]) || IsMe(cptr))
+#ifdef USE_POLL
+		pfd = &pollfds[i];
+		v = get_client_by_pollfd(pfd->fd);
+		if (v < 0)
+			continue; /* possible with resolver */
+		cptr = local[v];
+#else
+		cptr = local[i];
+#endif
+		if (!cptr || IsMe(cptr))
 			continue;
 
+#ifdef USE_POLL
+		if (pfd->revents & POLLOUT)
+#else
 		if (FD_ISSET(cptr->fd, &write_set))
+#endif
 		{
 			int  write_err = 0;
 			/*
@@ -1844,11 +2116,16 @@ int  read_message(time_t delay, fdlist *listp)
 			if (IsDead(cptr) || write_err)
 			{
 deadsocket:
+#ifdef USE_POLL
+				if (pfd->revents & POLLIN)
+					--nfds;
+#else
 				if (FD_ISSET(cptr->fd, &read_set))
 				{
 					nfds--;
 					FD_CLR(cptr->fd, &read_set);
 				}
+#endif
 				(void)exit_client(cptr, cptr, &me,
 				    ((sockerr = get_sockerr(cptr))
 				    ? STRERROR(sockerr) : "Client exited"));
@@ -1856,22 +2133,29 @@ deadsocket:
 			}
 		}
 		length = 1;	/* for fall through case */
-		/* Note: these DoingDNS/DoingAuth checks are here because of a
-		 * filedescriptor race condition, so don't remove them without
-		 * being sure that has been fixed. -- Syzop
+
+#ifdef USE_POLL
+		if (pfd->revents & POLLIN)
+#else
+		if (FD_ISSET(cptr->fd, &read_set))
+#endif
+			length = read_packet(cptr, 1);
+		/* If we don't have anything to read and we have any recvQ, then
+		 * read_packet must still be called, but this time with the 2nd parameter
+		 * being 0 (=don't read). This is so we check if anything needs to
+		 * be dequeued from the receive queue (due to fake lag). -- Syzop
 		 */
-		if ((!NoNewLine(cptr) || FD_ISSET(cptr->fd, &read_set)) &&
-		    !(DoingDNS(cptr) || DoingAuth(cptr))
-#ifdef USE_SSL
-			&& 
-			!(IsSSLHandshake(cptr))
-#endif		
-			)
-			length = read_packet(cptr, &read_set);
+		else if (DBufLength(&cptr->recvQ) > 0)
+			length = read_packet(cptr, 0);
+
 #ifdef USE_SSL
 		if ((length != FLUSH_BUFFER) && (cptr->ssl != NULL) && 
 			IsSSLHandshake(cptr) &&
+#ifdef USE_POLL
+			pfd->revents & POLLIN)
+#else
 			FD_ISSET(cptr->fd, &read_set))
+#endif
 		{
 			if (!SSL_is_init_finished(cptr->ssl))
 			{
@@ -1907,8 +2191,13 @@ deadsocket:
 			flush_connections(cptr);
 		if ((length != FLUSH_BUFFER) && IsDead(cptr))
 			goto deadsocket;
-		if ((length > 0) && (cptr->fd >= 0) && !FD_ISSET(cptr->fd, &read_set))
-			continue;
+		if ((length > 0) && (cptr->fd >= 0))
+#ifdef USE_POLL
+			if (!(pfd->revents & POLLIN))
+#else
+			if (!FD_ISSET(cptr->fd, &read_set))
+#endif
+				continue;
 		nfds--;
 		readcalls++;
 		if (length > 0)
