@@ -224,25 +224,27 @@ char *num = NULL;
 
 /*
  * send_proto:
- * sends PROTOCTL message to server, taking care of whether ZIP
+ * Sends PROTOCTL message to server, taking care of whether ZIP
  * should be enabled or not.
+ * Now split up into multiple PROTOCTL messages (again), since we have
+ * too many for a single line. If this breaks your services because
+ * you fail to maintain PROTOCTL state, then fix them!
  */
 void send_proto(aClient *cptr, ConfigItem_link *aconf)
 {
 char buf[1024];
 
-	sprintf(buf, "CHANMODES=%s%s,%s%s,%s%s,%s%s NICKCHARS=%s",
+	/* First line */
+	sendto_one(cptr, "PROTOCTL %s", PROTOCTL_SERVER);
+
+	/* Second line */
+	sprintf(buf, "CHANMODES=%s%s,%s%s,%s%s,%s%s NICKCHARS=%s MLOCK",
 		CHPAR1, EXPAR1, CHPAR2, EXPAR2, CHPAR3, EXPAR3, CHPAR4, EXPAR4, langsinuse);
 #ifdef ZIP_LINKS
 	if (aconf->options & CONNECT_ZIP)
-	{
-		sendto_one(cptr, "PROTOCTL %s ZIP %s", PROTOCTL_SERVER, buf);
-	} else {
-#endif
-		sendto_one(cptr, "PROTOCTL %s %s", PROTOCTL_SERVER, buf);
-#ifdef ZIP_LINKS
-	}
+		strcat(buf, " ZIP");
 #endif
+	sendto_one(cptr, "PROTOCTL %s", buf);
 }
 
 #ifndef IRCDTOTALVERSION
@@ -264,10 +266,11 @@ char *unrealinfo[] =
 	"* binki        <binki@unrealircd.com>",
 	"",
 	"Contributors:",
-	"* Stealth, tabrisnet, Bock, fbi",
+	"* nenolod, Adam, warg, Stealth, WolfSage, katsklaw, darkex, fspijkerman,",
+	"  fbi, Apocalypse",
 	"",
 	"RC Testers:",
-	"* LqidSkin, goldenwolf, katsklaw, nph, selten, Simplicity,",
+	"* therock247uk, katsklaw, nenolod, Han`, Adam, Cronus, warg, Crash,",
 	"  and everyone else who downloaded the release candidates.",
 	"",
 	"Past UnrealIRCd3.2* coders/contributors:",
